@@ -96,6 +96,7 @@ array_downcast_fn!(as_null_array, NullArray);
 array_downcast_fn!(as_struct_array, StructArray);
 array_downcast_fn!(as_union_array, UnionArray);
 array_downcast_fn!(as_map_array, MapArray);
+array_downcast_fn!(as_decimal_array, DecimalArray);
 
 #[cfg(test)]
 mod tests {
@@ -103,6 +104,18 @@ mod tests {
 
     use super::*;
 
+    #[test]
+    fn test_as_decimal_array_ref() {
+        let array: DecimalArray = vec![Some(123), None, Some(1111)]
+            .into_iter()
+            .collect::<DecimalArray>()
+            .with_precision_and_scale(10, 2)
+            .unwrap();
+        assert!(!as_decimal_array(&array).is_empty());
+        let result_decimal = as_decimal_array(&array);
+        assert_eq!(result_decimal, &array);
+    }
+
     #[test]
     fn test_as_primitive_array_ref() {
         let array: Int32Array = vec![1, 2, 3].into_iter().map(Some).collect();
