@@ -194,29 +194,16 @@ pub(crate) fn new_buffers(data_type: &DataType, capacity: usize) -> [MutableBuff
             MutableBuffer::new(capacity * mem::size_of::<u8>()),
             empty_buffer,
         ],
-        DataType::Union(_, _) => unimplemented!(),
-    }
-}
-
-/// Ensures that at least `min_size` elements of type `data_type` can
-/// be stored in a buffer of `buffer_size`.
-///
-/// `buffer_index` is used in error messages to identify which buffer
-/// had the invalid index
-fn ensure_size(
-    data_type: &DataType,
-    min_size: usize,
-    buffer_size: usize,
-    buffer_index: usize,
-) -> Result<()> {
-    // if min_size is zero, may not have buffers (e.g. NullArray)
-    if min_size > 0 && buffer_size < min_size {
-        Err(ArrowError::InvalidArgumentError(format!(
-            "Need at least {} bytes in buffers[{}] in array of type {:?}, but got {}",
-            buffer_size, buffer_index, data_type, min_size
-        )))
-    } else {
-        Ok(())
+        DataType::Union(_, mode) => {
+            let type_ids = MutableBuffer::new(capacity * mem::size_of::<i8>());
+            match mode {
+                UnionMode::Sparse => [type_ids, empty_buffer],
+                UnionMode::Dense => {
+                    let offsets = MutableBuffer::new(capacity * mem::size_of::<i32>());
+                    [type_ids, offsets]
+                }
+            }
+        }
     }
 }
 
@@ -228,11 +215,12 @@ pub(crate) fn into_buffers(
     buffer2: MutableBuffer,
 ) -> Vec<Buffer> {
     match data_type {
-        DataType::Null | DataType::Struct(_) => vec![],
+        DataType::Null | DataType::Struct(_) | DataType::FixedSizeList(_, _) => vec![],
         DataType::Utf8
         | DataType::Binary
         | DataType::LargeUtf8
-        | DataType::LargeBinary => vec![buffer1.into(), buffer2.into()],
+        | DataType::LargeBinary
+        | DataType::Union(_, _) => vec![buffer1.into(), buffer2.into()],
         _ => vec![buffer1.into()],
     }
 }
@@ -357,6 +345,27 @@ impl ArrayData {
         &self.data_type
     }
 
+    /// Updates the [DataType] of this ArrayData/
+    ///
+    /// panic's if the new DataType is not compatible with the
+    /// existing type.
+    ///
+    /// Note: currently only changing a [DataType::Decimal]s precision
+    /// and scale are supported
+    #[inline]
+    pub(crate) fn with_data_type(mut self, new_data_type: DataType) -> Self {
+        assert!(
+            matches!(self.data_type, DataType::Decimal(_, _)),
+            "only DecimalType is supported for existing type"
+        );
+        assert!(
+            matches!(new_data_type, DataType::Decimal(_, _)),
+            "only DecimalType is supported for new datatype"
+        );
+        self.data_type = new_data_type;
+        self
+    }
+
     /// Returns a slice of buffers for this array data
     pub fn buffers(&self) -> &[Buffer] {
         &self.buffers[..]
@@ -560,7 +569,10 @@ impl ArrayData {
             DataType::Map(field, _) => {
                 vec![Self::new_empty(field.data_type())]
             }
-            DataType::Union(_, _) => unimplemented!(),
+            DataType::Union(fields, _) => fields
+                .iter()
+                .map(|field| Self::new_empty(field.data_type()))
+                .collect(),
             DataType::Dictionary(_, data_type) => {
                 vec![Self::new_empty(data_type)]
             }
@@ -588,7 +600,7 @@ impl ArrayData {
     /// contents of the buffers (e.g. that all offsets for UTF8 arrays
     /// are within the bounds of the values buffer).
     ///
-    /// See [`validate_full`] to validate fully the offset content
+    /// See [ArrayData::validate_full] to validate fully the offset content
     /// and the validitiy of utf8 data
     pub fn validate(&self) -> Result<()> {
         // Need at least this mich space in each buffer
@@ -918,7 +930,7 @@ impl ArrayData {
     /// 3. All dictionary offsets are valid
     ///
     /// Does not (yet) check
-    /// 1. Union type_ids are valid (see https://github.com/apache/arrow-rs/issues/85)
+    /// 1. Union type_ids are valid see [#85](https://github.com/apache/arrow-rs/issues/85)
     /// Note calls `validate()` internally
     pub fn validate_full(&self) -> Result<()> {
         // Check all buffer sizes prior to looking at them more deeply in this function
@@ -1075,8 +1087,8 @@ impl ArrayData {
             })
     }
 
-    /// Ensures that all strings formed by the offsets in buffers[0]
-    /// into buffers[1] are valid utf8 sequences
+    /// Ensures that all strings formed by the offsets in `buffers[0]`
+    /// into `buffers[1]` are valid utf8 sequences
     fn validate_utf8<T>(&self) -> Result<()>
     where
         T: ArrowNativeType + std::convert::TryInto<usize> + num::Num + std::fmt::Display,
@@ -1099,7 +1111,7 @@ impl ArrayData {
         )
     }
 
-    /// Ensures that all offsets in buffers[0] into buffers[1] are
+    /// Ensures that all offsets in `buffers[0]` into `buffers[1]` are
     /// between `0` and `offset_limit`
     fn validate_offsets_full<T>(&self, offset_limit: usize) -> Result<()>
     where
@@ -1333,6 +1345,7 @@ enum BufferSpec {
     BitMap,
     /// Buffer is always null. Unused currently in Rust implementation,
     /// (used in C++ for Union type)
+    #[allow(dead_code)]
     AlwaysNull,
 }
 
